-- Import addon data
local _, addonData = ...;
-- Frames
local loginFrame = CreateFrame("Frame");
local questFrame = CreateFrame("Frame");
-- Events
local events = {};
local completedStorylines = {};

function events:GOSSIP_SHOW()
	--print("GOSSIP!");
	
	local activeQuests = GetNumGossipActiveQuests();
	local availableQuests = GetNumGossipAvailableQuests();
	
	if (availableQuests > 0) then
		-- Loop through all active quests
		for i = 1, availableQuests, 1 do
			SelectGossipAvailableQuest(i);
		end
	elseif (activeQuests > 0) then
		-- Loop through all available quests
		for i = 1, activeQuests, 1 do
			SelectGossipActiveQuest(i);
		end
	end
end

function events:QUEST_GREETING()
	--print(format("GREETING fired, available quests: %i", GetNumGossipAvailableQuests()));
	SelectGossipAvailableQuest(1);
end

function events:QUEST_DETAIL()
	--print("DETAILS!");
	AcceptQuest()
end

function events:QUEST_PROGRESS()
	--print("PROGRESS!");
	local isCompletable = IsQuestCompletable()
	
	--print(GetQuestID());
	
	if (isCompletable) then
		CompleteQuest()
	else
		DLU.createMessage("Quest couldn't be completed!");
	end
end

function events:QUEST_COMPLETE()
	local choices = GetNumQuestChoices();
	
	--print(format("Quest completion event raised! Amount of rewards: %i", choices));
	
	local choice = 1;
	if (choices < 2) then
		GetQuestReward(choice)
		--print("Quest completed!");
	end
	
	
	--[[
	if (choices > 1) then
		-- Find most suitable choice
	else
		GetQuestReward(choice)
	end
	]]
end

function events:QUESTLINE_UPDATE()
	local mapId = C_Map.GetBestMapForUnit("player");
	local storyAchievementID, storyMapID = C_QuestLog.GetZoneStoryInfo(mapId);
	if (storyAchievementID) then
		local mapInfo = C_Map.GetMapInfo(storyMapID);
		local numCriteria = GetAchievementNumCriteria(storyAchievementID);
		
		local completedCriteria = 0;
		for i = 1, numCriteria do
			local _, _, completed = GetAchievementCriteriaInfo(storyAchievementID, i);
			if ( completed ) then
				completedCriteria = completedCriteria + 1;
			end
		end
		
		if (numCriteria == completedCriteria) then
			DLU.createNotification("You have completed the storyline in this area.");
		end
	end
end

-- Login
loginFrame:RegisterEvent("PLAYER_LOGIN");
loginFrame:SetScript("OnEvent", function()
	
	if (DLUCharacterSettings.questingMode.value) then
		questFrame:SetScript("OnEvent", function(self, event, ...)
			events[event](self, ...);
		end);
		for k, v in pairs(events) do
			questFrame:RegisterEvent(k);
		end
	end
	
	loginFrame:UnregisterEvent("PLAYER_LOGIN")
end);